#pragma once

#include "Pokitto.h"
#include "Camera.h"

class Layer
{

private:
    Camera* cam;
    const MapConfig* mapConfig;
    const Tile* mapTiles;
    const TileAnimation* mapAnimation;
    TileDamage* mapDamage;
    const TileAction* mapAction;

    int8_t frame=0;
    int8_t frameSub=0;
    int8_t frameMax;
    int8_t idAnim;

public:

    typedef struct
    {
        uint8_t Sprite=0;
        PointT<int16_t> Position;
        PointT<int16_t> PositionDelta;
        RectangleT<int16_t> HitBox;
        bool FippedH=false;
        bool FippedV=false;
        bool Animated=false;
        bool Solid=false;
        bool OneWay=false;
        bool Destructable=false;
        bool Actioned=false;
        bool Damaging=false;
        bool Alive=false;
        bool Action=false;
    } TileExp;

    TileExp* screenTiles;

    Layer(Camera* cam,const MapConfig* mapConfig,const Tile* mapTiles,
          const TileAnimation* mapAnimation,TileDamage* mapDamage, const TileAction* mapAction,
          TileExp* screenTiles)
    {
        this->cam=cam;
        this->mapConfig=mapConfig;
        this->mapTiles=mapTiles;
        this->mapAnimation=mapAnimation;
        this->mapDamage=mapDamage;
        this->mapAction=mapAction;
        this->screenTiles=screenTiles;
    }


    void update()
    {
        cam->update();

        frameSub++;
        if(frameSub>10)
        {
            frame++;
            frameSub=0;
        }
    }

    Camera* getCamera()
    {
        return cam;
    }

    void setCameraTarget(int16_t xt,int16_t yt)
    {
        cam->setTarget(xt,yt);
    }

    bool isAlive(int16_t xc,int16_t yc)
    {
        return true;
    }

    Tile getTile(int16_t col,int16_t row)
    {
        return mapTiles[(row*mapConfig->Columns)+col];
    }

    TileExp getTileExp(int16_t col,int16_t row,int16_t colScreen,int16_t rowScreen)
    {
        Tile t=getTile(col,row);
        TileExp tExp=TileExp();
        tExp.Sprite=t.Sprite;

        //Init defaults
        tExp.Alive=false;
        tExp.Position.x = colScreen*CELL_WIDTH;
        tExp.Position.y = rowScreen*CELL_HEIGHT;
        tExp.PositionDelta.x=0;
        tExp.PositionDelta.y=0;
        tExp.HitBox.Set(0,0,0,0);

        if(tExp.Sprite!=0)
        {
            tExp.FippedH=t.Properties[0];
            tExp.FippedV=t.Properties[1];
            tExp.Animated=t.Properties[2];
            tExp.Solid=t.Properties[3];
            tExp.OneWay=t.Properties[4];
            tExp.Destructable=t.Properties[5];
            tExp.Actioned=t.Properties[6];
            tExp.Damaging=t.Properties[7];

            tExp.Alive=true;

            //Destructible? Is Alive?
            if(mapDamage!=nullptr && tExp.Destructable)
            {
                for(uint16_t i=0; i<mapConfig->DamageTilesCount; i++)
                {
                    TileDamage td=this->mapDamage[i];
                    if(td.Column==col && td.Row==row)
                    {
                        tExp.Alive=td.Life>0;
                        //Not alive --> Not Solid
                        if(!tExp.Alive)
                        {
                            tExp.Solid=false;
                        }
                        break;
                    }
                }
            }

            //Animated? -> Change tiles
            if(mapAnimation!=nullptr && tExp.Animated && tExp.Alive)
            {
                for(uint16_t i=0; i<mapConfig->AnimationTilesCount; i++)
                {
                    TileAnimation ta=this->mapAnimation[i];
                    if(ta.Sprite==t.Sprite && ta.Frame==(frame%ta.MaxFrame))
                    {
                        tExp.Sprite=ta.SpriteAnimated;
                        break;
                    }
                }
            }

            //Action?
            if(mapAction!=nullptr && tExp.Actioned && tExp.Alive)
            {
                for(uint16_t i=0; i<mapConfig->ActionTilesCount; i++)
                {
                    TileAction ta=this->mapAction[i];
                    if(ta.Column==col && ta.Row==row)
                    {
                        if(ta.ActionType == 2)
                        {
                            tExp.PositionDelta.x = abs(frame % (2*ta.ActionParameter) - ta.ActionParameter);
                            tExp.Position.x += tExp.PositionDelta.x;
                        }
                        break;
                    }
                }
            }

            //Collision box
            if(tExp.Solid)
            {
                tExp.HitBox.x1=(col*CELL_WIDTH)+tExp.PositionDelta.x;
                tExp.HitBox.y1=(row*CELL_HEIGHT)+tExp.PositionDelta.y;
                tExp.HitBox.x2=tExp.HitBox.x1+CELL_HEIGHT;
                tExp.HitBox.y2=tExp.HitBox.y1+CELL_WIDTH;
            }
            else
                tExp.HitBox.Set(0,0,0,0);
        }
        return tExp;
    }

    void damageTile(int16_t col,int16_t row,int8_t damage)
    {
        for(uint16_t i=0; i<mapConfig->DamageTilesCount; i++)
        {
            if(mapDamage[i].Column==col && mapDamage[i].Row==row)
            {
                if(mapDamage[i].Life>0)
                {
                    mapDamage[i].Life-=damage;
                    break;
                }
            }
        }
    }

    void draw()
    {
        TileExp tExp;

        //Calculate
        int xCellStart = (cam->getX()/cam->getZ())/CELL_WIDTH;
        int yCellStart = (cam->getY()/cam->getZ())/CELL_HEIGHT;

        int xCellRem = (cam->getX()/cam->getZ())%CELL_WIDTH;
        int yCellRem = (cam->getY()/cam->getZ())%CELL_HEIGHT;

        int cc=xCellStart;
        int rr=yCellStart;
        for(int r=0; r<12; r++,rr++)//11 rows +1
        {
            cc=xCellStart;
            for(int c=0; c<15; c++,cc++)//14 columns + 1
            {
                tExp=getTileExp(cc%mapConfig->Columns,rr%mapConfig->Rows,c,r);
                //If request save extended tiles to passed array
                if(screenTiles!=nullptr)
                {
                    *(screenTiles+(r*15)+c) = tExp;
                }
                //Draw tiles if need
                if(tExp.Sprite!=0 && cc>0 && rr>0 && tExp.Alive)
                {
                    if(tExp.FippedH)
                        d.drawBitmapXFlipped(tExp.Position.x-xCellRem,
                                             tExp.Position.y-yCellRem,
                                             sprites[tExp.Sprite]);
                    else
                        d.drawBitmap(tExp.Position.x-xCellRem,
                                     tExp.Position.y-yCellRem,
                                     sprites[tExp.Sprite]);
                }
            }
        }
    }
};


